import sphinx_rtd_theme
import os
from sphinx_markdown_parser.parser import MarkdownParser
from datetime import datetime
#from recommonmark.transform import AutoStructify

source_parsers = {
    '.md': 'CommonMarkParser',
}

# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options. For a full
# list see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
# import os
# import sys
# sys.path.insert(0, os.path.abspath('.'))


# -- Project information -----------------------------------------------------

project = 'Documentation Veremes'
copyright = 'https://www.veremes.com'
author = 'Veremes'

# The full version, including alpha/beta/rc tags
release = '2'
version = "1.0.0"

# Python variable available in HTML template
html_context = {
    'actual_year_copyright' : str(datetime.now().year) + ', <a href="https://www.veremes.com" target="_blank">https://www.veremes.com</a>.'
}

# -- General configuration ---------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    'sphinx_rtd_theme',
    'sphinx.ext.autodoc',
    'sphinx.ext.doctest',
    'sphinx.ext.intersphinx',
    'sphinx.ext.todo',
    'sphinx.ext.coverage',
    'sphinx.ext.mathjax',
    'sphinx.ext.autosectionlabel',
    'myst_parser',
    'sphinx.ext.viewcode',
]

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#
# This is also used if you do content translation via gettext catalogs.
# Usually you set "language" from the command line for these cases.
language = 'fr'

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = []

html_css_files = [
    'custom.css',
]

html_show_sourcelink = False

smartquotes = False


# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
# Theme used
html_theme = 'sphinx_rtd_theme'

# Favicon of the website (the icon located at the top)
html_favicon = 'images/favicon.ico'

# Last time documentation was generated in format Day Month Year(ex: 17 Mai 2021)
html_last_updated_fmt = '%-d %B %Y'

# Logo of the website, what appear at the top of the summary, under the name of the project
# html_logo = '_static/code.png'
html_logo = 'images/logo-menu.png'

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ['_static']


# These paths are either relative to html_static_path
# or fully qualified paths (eg. https://...)
html_css_files = [
    'css/custom.css',
]

# All the foremat supported
source_suffix = {
    '.rst': 'restructuredtext',
    '.txt': 'markdown',
    '.md': 'markdown',
}

# Masque "Compilé avec Sphinx en utilisant un thème fourni par Read the Docs."
html_show_sphinx = True

# Options for the themen you can find how it work at https://sphinx-rtd-theme.readthedocs.io/en/stable/configuring.html
html_theme_options = {
    'canonical_url': '',
    'analytics_id': 'UA-25689082-1',  # Provided by Google in your dashboard
    'logo_only': False,
    'display_version': False,
    'prev_next_buttons_location': 'bottom',
    'style_external_links': False,
    # 'vcs_pageview_mode': '',
    # 'style_nav_header_background': 'cyan',
    # Toc options
    'collapse_navigation': True,
    'sticky_navigation': True,
    'navigation_depth': 4,
    'includehidden': True,
    'titles_only': False
}
