# Usages avancés

Cette partie de la documentation s'adresse principalement aux maintainers et responsables technique.

## Force Push

Dans certains cas Gitkraken vous proposera de réaliser un force push. 
Il faut savoir que cette action est lourde de conséquences et il faut bien être conscient de ce que vous faites en cliquant sur ce bouton.

Au moment de réaliser un force push vous considérez que votre dépôt est correct et que c'est le dépôt distant qui est dans l'erreur, vous écrasez donc l'historique du serveur distant avec le votre **au risque de perdre le commit qui vous a imposé de force push**. De plus vous allez reporter la gestion de conflits sur le développeur qui a push les commits que vous avez écrasé, ou le forcer à réaliser un `git reset --hard` voir pire. 

Dans la majorité des cas le force push n'est pas la solution et posera souvent plus de problèmes qu'autre chose.

Le plus souvent un simple pull avant de push (et une petite gestion de conflits) vous assurera de ne pas avoir à force push. Pour les binaires ou les projets FME c'est plus complexe, il faut essayer de reporter les modifications ou choisir un candidat viable à la résolution de conflit.

**Si vous avez perdu de la donnée** suite à un force push, c'est récupérable si vous n'êtes pas seul sur le dépôt, et qu'un autre utilisateur à chekout la branche et qu'il n'a pas pull vos modifications. Il pourra alors gérer le conflit et force push à son tour pour corriger le problème.

```{Warning}
  Dans le doute, toujours appeler quelqu'un qui maitrise Git. On préférera toujours rattraper une petite bêtise, que la grosse bêtise qui découlera des quinze tentatives de résolution intermédiaires qui n'auront pas fonctioné avant de nous appeler. Git est un élément fondamental dans la tenue de notre code, il est primordial que les dépôts restent toujours propres, fonctionnels et que l'on ne perde pas de code.
```

## Git Bisect

Git intègre un outil de recherche d'erreur qui utilise la méthode de [recherche par bissection](https://fr.wikipedia.org/wiki/M%C3%A9thode_de_dichotomie).
Une recherche par bissection, consiste à prendre un ensemble, le couper en deux et déterminer quel ensemble contient ce qu'on cherche, puis on réitère l'opération sur le sous-ensemble en question jusqu'à trouver l'élément de l'ensemble d'origine que l'on cherche.


Voici comment l'algorithme va agir : 

![git bisect fonctionnement](/images/git/git_bisect_fonctionnement.svg)

La commande va faire glisser le commit référencé par le HEAD, entre le commit actuel et un commit identifié comme sain (le commit sain n'est pas obligatoire, git bisect peut aussi l'identifier lors des premiers déplacements).
La bissection va glisser sur un commit situé légèrement au-delà de la médiane (environ à 50-60% de la distance entre le commit actuel et le commit cible) et va demander si le bug est présent ou non.
Si le bug est présent alors l'algorithme ira chercher un commit plus ancien, sinon il prendra un commit plus récent. C'est à vous de dire à git si le bug est présent ou non via les commandes `git bisect good` et `git bisect bad`. La commande `git bisect skip` permet de changer de commit sans répondre à la question en cas de doute.
Dès que deux commits successifs présenteront un cas sain et un cas d'erreur, alors le commit à l'origine du bug sera identifié.

Exemple : Pour cet exemple j'ai répondu aléatoirement sur les commits situés entre mon HEAD next_app_dtnet et son 50éme parent direct (ce qui représente 283 commits avec les merges)

![git bisect example](/images/git/git_bisect_example.png)

Si la présence du bug est testable avec un script shell, il sera alors possible de passer directement le script à la commande git bisect qui pourra identifier le bug très rapidement.

Exemple : npm build ne fonctionne plus depuis le dernier tag 
```shell
# Lance la recherche entre deux commits
git bisect start HEAD 2022.XX.XX
# Remplacer ma commande par un script sh sera surement plus efficace
# Recherche l'erreur
git bisect run sh -c "cd client && npm run build --prod"
# Quitte la recherche
git bisect reset

```

## Git Rebase et interactive rebase

La commande rebase permet de réécrire l'historique. Rebase une branche sur une autre va en fait réécrire l'historique des deux branches pour les rendre fast-forward.
C'est un usage très rare de cette commande, vu que dans notre cas la majorité des fusions de branches passent par des merge requests et donc des merges classiques.

En revanche l'interactive rebase est un outil très utile pour réorganiser ses commits avant de push.

![Interactive rebase gk usecase](/images/git/gk_ex_rebase.png)

Je peux alors réaliser un clique droit sur le commit HEAD de la branche distante, et sélectionner `Interactive rebase [X] children of [sha1]`.

en faisant ça, j'arrive sur l'interface de rebase interactif :

![Interactive rebase gk](/images/git/gk_interactive_rebase_ihm.png)

Plusieurs actions sont possibles : 
- **pick** : Laisse le commit dans l'état 
- **reword** : Permet de modifier le commentaire
- **squash** : Permet de fusionner deux commits adjacents
- **drop** : Permet de supprimer un commit
- **move** : Permet de déplacer un commit

```{Tip}
  Certaines de ces actions sont réalisables via le menu contextuel en faisant un clique droit sur un commit ou un ensemble de commits sans réaliser un git rebase complet.
```

![Interactive rebase gk update](/images/git/gk_interactive_rebase_update.png)


Dans ce cas-là j'ai déplacé mon commit pour corriger la faute d'orthographe, j'améliore le message du second commit, et je fusionne mes deux derniers commits en changeant le message qui sera affiché sur le commentaire. Pour finaliser je dois cliquer sur le bouton  `start rebase`, ce qui modifiera l'arborescence des commits.


```{Warning}
  Réaliser un intéractive rebase sur des commits déjà poussés sur le dépôt distant provoquera systématiquement l'apparition d'un force push, ce qui risque de corrompre l'historique global du dépôt. Si vous travaillez seul sur la branche rebase et que vous maitrisez bien ce que vous faites c'est sans risque.
```

## Git Blame, Git History

Via GitKraken la récupération du git blame ou du git history se fait en même temps.

La récupération de l'historique lié à un fichier, c'est en réalité la récupération des commits qui sont en lien avec ce fichier. \
Le git blame sur un fichier permet de voir quel est le dernier commit à avoir agit sur chaque ligne.

Pour ouvrir l'historique, il suffit de réaliser un clique droit sur le fichier que l'on souhaite analyser et de choisir `File History` ou `File Blame`, qui ouvrira le même composant.

```{Tip}
  Si vous ne voulez pas chercher votre fichier dans la liste des commits pour gagner du temps. Il vous suffit de cocher la checkbox au-dessus de l'arborescence, `View all files` pour trouver facilement votre fichier, L'historique partira du premier commit impactant ce fichier, antérieur au commit sélectionné.
```


![GitKraken blame](/images/git/gk_blame.png)

Sur la partie de gauche, on retrouve donc l'historique des commits sur le fichier, et sur la partie de droite le blame.
en cliquant dans l'historique on peut voir d'autres versions du fichier si nécessaire (le blame évoluera en même temps). 

## Branche courante dans l'invite de commande

Sous Linux, il faut ajouter le script `~/.git-prompt.sh` disponible sur [Github](https://github.com/git/git/blob/master/contrib/completion/git-prompt.sh)


Il faut éditer votre fichier `~/.bashrc` et modifier la variable PS1

```shell
# Permet de charger les variable nécessaire pour la PS1
if [ -f ~/.git-prompt.sh ]; then
    GIT_PS1_SHOWDIRTYSTATE=true
    GIT_PS1_SHOWUNTRACKEDFILES=true
    GIT_PS1_SHOWSTASHSTATE=true
    GIT_PS1_SHOWUPSTREAM=true
    GIT_PS1_SHOWCOLORHINTS=true
    . ~/.git-prompt.sh
fi

# Mofidier les lignes commentés qui sont surement déjà présentent dans votre bashrc par la deuxième ligne
if [ "$color_prompt" = yes ]; then
#    PS1='${debian_chroot:+($debian_chroot)}\[\033[01;32m\]\u@\h\[\033[00m\]:\[\033[01;34m\]\w\[\033[00m\]\$ '
    PS1='${debian_chroot:+($debian_chroot)}\[\033[01;32m\]\u@\h\[\033[00m\]:\[\033[01;34m\]\w\[\033[00m\]\[\033[0;31m\]$(__git_ps1 " (%s)" )\[\033[00m\]\$ '
else
#    PS1='${debian_chroot:+($debian_chroot)}\u@\h:\w\$ '
    PS1='${debian_chroot:+($debian_chroot)}\u@\h:\w$(__git_ps1 " (%s)" )\$ '
fi
unset color_prompt force_color_prompt
```

Lorque vous serez dans le dossier d'un dépôt Git, la branche et l'état du dépôt apparaitra alors au niveau de la CLI : 

![rendu CLI](/images/git/branch_in_cli.png)

La gestion de l'état se fait via une liste de symbole : 
- \* : modification en cours
- \+ : Fichier présent dans la zone de cache
- \% : Présence d'un fichier non connu et non ignoré
- < : En retard sur la branche distante upstream
- \> : En avance sur la branche distante upstream
- = : La branche distante upstream est au même niveau
- $ : Présence d'un stash dans le dépôt


Pour Windows, Git Bash le fera de lui-même, mais si besoin il est possible de configurer le Powershell pour obtenir le même rendu. Une documentation décrit la procédure sur [Git Scm](https://git-scm.com/book/uz/v2/Appendix-A%3A-Git-in-Other-Environments-Git-in-Powershell)
## Fork

La documentation relative au fork sur Gitlab est disponible via [ce lien](https://docs.gitlab.com/ee/user/project/repository/forking_workflow.html#project-forking-workflow)

## CI/CD

Gitlab permet de déclencher des scripts suite à des push, c'est très intéressant dans le cadre de développement en mode DevOps. On parle d'intégration continue (CI) et de déploiement continu (CD).

La documentation spécifique à Gitlab sur la conception de pipeline CI/CD est disponible via [ce lien](https://docs.gitlab.com/ee/ci/). 
Pour la documentation, attention toutes les fonctionnalités ne sont pas compatibles avec notre version et notre édition de Gitlab.

Nous nous en servons, majoritairement pour vérifier qu'un certain niveau de qualité/propreté du code est maintenu en continu sur les dépôts, ou pour publier la documentation automatiquement.

La fonctionnalité doit être activée sur le dépôt via la configuration du projet sur Gitlab.

Une fois la fonctionnalité active, il faut ajouter un fichier `.gitlab-ci.yml` à la racine du dépôt.

```{Tip}
  Pour créer un CI/CD petit conseil, partez d'une branche master ou proche de master, puis une fois terminer mergez cette branche dans master et master dans les branches type next_version rapidement de façon à propager le CI/CD rapidement et s'éviter des conflits ou des comportements étranges du CI/CD sur GitLab.
```

exemple de CI/CD pour le fichier yml : Génération de la documentation de développement (uniquement si modifié sur next_version)
```yaml
# Image docket à utiliser pour lancer el CI/CD
image: veremes/sphinx-doc:latest

# Variable utilisable dans le script
variables:
  S3_BUCKET_NAME_DEV_fr: "documentation-dev.veremes.net/dev/fr"

# Deploiement documentation francaise
deploy-gtf-fr-documentation-dev :
  stage: deploy
  script:
    - sphinx-build -b html source build
    - aws s3 cp build/ s3://$S3_BUCKET_NAME_DEV_fr/ --recursive
  tags:
    - deployment
  only:
    refs:
      - next_version
    changes:
      - source/**/*
```